#!/usr/bin/perl -w

# check firefly fortune file for bad formatting and such.

$maxcols = 79; # no line in the file may be longer than this (not counting the \n).
$maxlines = 23 ; # no fortune may be longer than this many lines.

# implemented checks:
# - no \r line endings
# - all fortunes must be at least one line long, and less than $maxlines
# - first line of a fortune must not be blank
# - no tabs
# - no line may be longer than $maxcols
# - punctuation goes inside quotes: "like this." and not "like this".
# - separator lines (%) must consist only of the % (1 character long)
# - no trailing whitespace (including \r)
# - no leading whitespace except for last (attribution) line
# - no line may consist of only whitespace (blank lines must be empty).
# - only printable ASCII allowed
# - no blank line just before last (attribution) line
# - must actually have attribution line, with leading whitespace, --, one space
# - attribution line must be exactly 72 characters.
# - first and last lines of the file must not be separators

sub err {
	warn "$file:$.: $_[0]\n";
}

# reassign $. to the starting line number of this fortune, since that's
# what should appear in the error message. put it back, when done. could
# have used 'local $_ = $_[0]' but someday local might get removed from
# perl, as it's considered kinda archaic.
sub check_fortune {
	my $olddot = $.;
	$. = $_[0];
	shift;

	if(@_ == 0) {
		err "empty fortune.";
	} elsif(@_ == 1) {
		err "fortune is only one line.";
	} else {
		err "fortune is >$maxlines lines." if @_ > $maxlines;
		err "first line is blank." if $_[0] =~ /^\s*$/;

		for($_[-1]) {
			if(/^\s+--\s\S/) {
				err "attribution line not 72 chars." unless length == 72;
			} else {
				err "no (or malformed) attribution line.";
			}
		}
		err "blank line before attribution, remove." if($_[-2] =~ /^\s*$/);
	}

	$. = $olddot;
}

sub check_line {
	for($_[0]) {
		err "line too long (>$maxcols chars)." if length > $maxcols;
		err "hard tab; replace with spaces." if /\t/;
		err "punctuation outside quotes." if /"[.?!]$/;
		err "nonprintable character(s)." if /[\x00-\x08\x0a-\x1f\x7f]/;
		err "non-ASCII character(s): 0x" . sprintf("%02x", ord($&)) if /[^\x20-\x7e]/;
		if(/^\s+$/) {
			err "'blank' line has whitespace.";
		} elsif(/\s+$/) {
			err "trailing whitespace.";
		} elsif(/^\s+/) {
			if(!/^\s+--/) {
				err "leading whitespace on non-attribution line.";
			}
		}
	}
}

## main()
die "usage: $0 fortune-file\n" unless @ARGV == 1;
$file = $ARGV[0];
@fortune = ();
$fortune_start = 1;
while(<>) {
	chomp;
	die "$file has DOS CRLF line endings, fix it.\n" if /\r/;

	if(/^(\s*)%(\s*)$/) {
		err("bad % separator line (has whitespace).") unless $_ eq '%';
		err("first line of file is a % separator (remove it).") if $. == 1;
		check_fortune($fortune_start, @fortune);
		@fortune = ();
		$fortune_start = $. + 1;
		next;
	}
	push @fortune, $_;
	check_line($_);
}
check_fortune($fortune_start, @fortune);
err("last line of file is a % separator (remove it).") if @fortune == 0;
